# pylint: disable = missing-docstring, invalid-name
"""
Tests that the time range string in the filename is generated correctly.
"""
import numpy as np
import os
import unittest

import cmor


class TestHasCurDatasetAttribute(unittest.TestCase):
    """
    Tests for ``cmor.has_cur_dataset_attribute``.
    """

    def setUp(self):
        cmor.setup(inpath='Test',
                   netcdf_file_action=cmor.CMOR_REPLACE_4)

        cmor.dataset_json("Test/CMOR_input_example.json")

        self.path = None

    def test_decadal(self):
        table = 'Tables/CMIP6_Odec.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             # we're using the 360 day calendar
             'coord_vals': [5 * 360, 15 * 360],
             'cell_bounds': [[0, 10 * 360], [10 * 360, 20 * 360]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('hfds', 'W m-2', axis_ids, positive='up')
        cmor.write(varid, [5., 10.])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'hfds_Odec_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_2005-2015.nc')

    def test_yr(self):
        table = 'Tables/CMIP6_Eyr.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             'coord_vals': np.array([182.5, 547.5]),
             'cell_bounds': [[0, 365], [365, 730]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('baresoilFrac', '%', axis_ids)
        cmor.write(varid, [40, 60])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'baresoilFrac_Eyr_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_2000-2001.nc')

    def test_mon(self):
        table = 'Tables/CMIP6_Amon.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             'coord_vals': np.array([15, 45]),
             'cell_bounds': [[0, 30], [30, 60]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('ts', 'K', axis_ids)
        cmor.write(varid, [273, 274])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'ts_Amon_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001-200002.nc')

    def test_monclim(self):
        table = 'Tables/CMIP6_Oclim.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time2',
             'units': 'days since 2000-01-01 00:00:00',
             'coord_vals': np.array([15, 45]),
             'cell_bounds': [[0, 30], [30, 60]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('difmxybo2d', 'm4 s-1', axis_ids)
        cmor.write(varid, [273, 274])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'difmxybo_Oclim_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001-200002-clim.nc')

    def test_day(self):
        table = 'Tables/CMIP6_Eday.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time',
             'units': 'hours since 2000-01-01 00:00:00',
             'coord_vals': np.array([12, 36]),
             'cell_bounds': [[0, 24], [24, 48]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('ts', 'K', axis_ids)
        cmor.write(varid, [303, 305])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'ts_Eday_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_20000101-20000102.nc')

    def test_6hr(self):
        table = 'Tables/CMIP6_6hrLev.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
             'units': 'hours since 2000-01-01 00:00:00',
             'coord_vals': np.array([3, 9]),
             'cell_bounds': [[0, 6], [6, 12]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('ps', 'Pa', axis_ids)
        cmor.write(varid, [99300, 100000])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'ps_6hrLev_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001010300-200001010900.nc')

    def test_3hr(self):
        table = 'Tables/CMIP6_3hr.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
             'units': 'hours since 2000-01-01 00:00:00',
             'coord_vals': np.array([1.5, 4.5]),
             'cell_bounds': [[0, 3], [3, 6]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('ps', 'Pa', axis_ids)
        cmor.write(varid, [99300, 100000])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'ps_3hr_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001010130-200001010430.nc')

    def test_1hr(self):
        table = 'Tables/CMIP6_E1hr.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
            'units': 'minutes since 2000-01-01 00:00:00',
            'coord_vals': np.array([30, 90]),
            'cell_bounds': [[0, 60], [60, 120]]},
            {'table_entry': 'latitude',
            'units': 'degrees_north',
            'coord_vals': [0],
            'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
            'units': 'degrees_east',
            'coord_vals': [90],
            'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('psl', 'Pa', axis_ids)
        cmor.write(varid, [99300, 100000])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'psl_E1hr_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001010030-200001010130.nc')

    def test_1hrclimmon(self):
        table = 'Tables/CMIP6_E1hrClimMon.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time3',
             'units': 'minutes since 2000-01-01 00:00:00',
             'coord_vals': np.array([12.5, 37.5]),
             'cell_bounds': [[0., 25.], [25., 50.]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('rlut', 'W m-2', axis_ids, positive='up')
        cmor.write(varid, [186, 190])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'rlut_E1hrClimMon_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001010000-200001010050-clim.nc')

    def test_subhr(self):
        table = 'Tables/CMIP6_Esubhr.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
             'units': 'seconds since 2000-01-01 00:00:00',
             'coord_vals': np.array([750, 2250]),
             'cell_bounds': [[0, 1500], [1500, 3000]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('pr', 'kg m-2 s-1', axis_ids)
        cmor.write(varid, [0.000005, 0.000006])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'pr_Esubhr_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_20000101001230-20000101003730.nc')

    def test_day_rounds(self):
        table = 'Tables/CMIP6_Eday.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time',
             'units': 'days since 1960-01-01 00:00:00',
             'coord_vals': np.array([0.9999999, 1.9999999]),
             'cell_bounds': [[0.9999999 - 0.5, 0.9999999 + 0.5],
                             [0.9999999 + 0.5, 0.9999999 + 1.5]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('ts', 'K', axis_ids)
        cmor.write(varid, [303, 305])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'ts_Eday_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_19600102-19600103.nc')

    def test_hour_rounds_midnight(self):
        table = 'Tables/CMIP6_6hrLev.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
             'units': 'hours since 2000-01-01 00:00:00',
             'coord_vals': np.array([23.99999, 30]),
             'cell_bounds': [[20.99999, 27], [27, 33]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('ps', 'Pa', axis_ids)
        cmor.write(varid, [99300, 100000])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'ps_6hrLev_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001020000-200001020600.nc')

    def test_hour_rounds_minutes(self):
        table = 'Tables/CMIP6_E1hr.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
             'units': 'minutes since 2000-01-01 00:00:00',
             'coord_vals': np.array([12.6, 37.4])},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('psl', 'Pa', axis_ids)
        cmor.write(varid, [99300, 100000])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'psl_E1hr_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001010013-200001010037.nc')

    def test_subhr_rounds_seconds(self):
        table = 'Tables/CMIP6_Esubhr.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time1',
             'units': 'seconds since 2000-01-01 00:00:00',
             'coord_vals': np.array([750.4, 2250.6])},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('pr', 'kg m-2 s-1', axis_ids)
        cmor.write(varid, [0.000005, 0.000006])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'pr_Esubhr_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_20000101001230-20000101003731.nc')

    def test_monclim_rounding_start_time(self):
        table = 'Tables/CMIP6_Oclim.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time2',
             'units': 'days since 2000-01-01 00:00:00',
             'coord_vals': np.array([15, 45]),
             'cell_bounds': [[-0.00001, 31], [31, 59]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('difmxybo2d', 'm4 s-1', axis_ids)
        cmor.write(varid, [273, 274])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'difmxybo_Oclim_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001-200002-clim.nc')

    def test_monclim_rounding_end_time(self):
        """
        60 days since 2000-01-01 is 2000-03-01 00:00:00 and so this test makes
        sure that an end date at the start of the month is displayed as the
        previous month.
        """
        table = 'Tables/CMIP6_Oclim.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'time2',
             'units': 'days since 2000-01-01 00:00:00',
             'coord_vals': np.array([15, 45]),
             'cell_bounds': [[0, 31], [31, 60]]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('difmxybo2d', 'm4 s-1', axis_ids)
        cmor.write(varid, [273, 274])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'difmxybo_Oclim_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn_200001-200002-clim.nc')

    def test_fx(self):
        table = 'Tables/CMIP6_fx.json'
        cmor.load_table(table)
        axes = [
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]}
        ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)
        varid = cmor.variable('orog', 'm', axis_ids)
        cmor.write(varid, [273, 274])
        self.path = cmor.close(varid, file_name=True)

        self.assertEqual(os.path.basename(self.path),
                         'orog_fx_PCMDI-test-1-0_piControl-withism_'
                         'r3i1p1f1_gn.nc')

#    def tearDown(self):
#        if self.path:
#            if os.path.exists(self.path):
#                os.remove(self.path)


if __name__ == '__main__':
    unittest.main()



import cmor
import numpy
import unittest


class TestCMIP6Plus(unittest.TestCase):

    def test_multiple_modeling_realms(self):
        ntimes_passed = 12

        self.assertEqual(
            cmor.setup(inpath='mip-cmor-tables/Tables',
                       netcdf_file_action=cmor.CMOR_REPLACE),
            0)
        self.assertEqual(cmor.dataset_json("Test/CMIP6Plus_user_input.json"),
                         0)

        self.assertEqual(cmor.load_table("MIP_OPmon.json"), 0)

        axes = [
                {
                    'table_entry': 'time',
                    'units': 'months since 2010-1-1',
                    'coord_vals': numpy.arange(0, ntimes_passed, 1),
                    'cell_bounds': numpy.arange(0, ntimes_passed+1, 1)
                },
                {
                    'table_entry': 'latitude',
                    'units': 'degrees_north',
                    'coord_vals': [0],
                    'cell_bounds': [-1, 1]
                },
                {
                    'table_entry': 'longitude',
                    'units': 'degrees_east',
                    'coord_vals': [90],
                    'cell_bounds': [89, 91]
                }
            ]

        axis_ids = [cmor.axis(**axis) for axis in axes]

        ivar = cmor.variable('hfsifrazil2d', units='W m-2', axis_ids=axis_ids)

        data = [280., ] * ntimes_passed

        self.assertEqual(
            cmor.write(ivar, data, ntimes_passed=ntimes_passed),
            0)

        self.assertFalse(cmor.has_cur_dataset_attribute('further_info_url'))

        realms = cmor.get_cur_dataset_attribute('realm')
        self.assertEqual(realms, "ocean seaIce")

        title = cmor.get_cur_dataset_attribute('title')
        self.assertEqual(title,
                         'HadGEM3-GC31-LL output prepared for CMIP6Plus')

        self.assertEqual(cmor.close(), 0)


if __name__ == '__main__':
    unittest.main()



import json
import numpy as np
import cmor
import unittest
import os

from netCDF4 import Dataset

DATASET_INFO = {
    "_AXIS_ENTRY_FILE": "CMIP6_coordinate_leadtime.json",
    "_FORMULA_VAR_FILE": "Tables/CMIP6_formula_terms.json",
    "_controlled_vocabulary_file": "Tables/CMIP6_CV.json",
    "activity_id": "CMIP",
    "branch_method": "standard",
    "branch_time_in_child": 30.0,
    "branch_time_in_parent": 10800.0,
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "experiment": "AMIP",
    "experiment_id": "amip",
    "forcing_index": "3",
    "further_info_url": "https://furtherinfo.es-doc.org/CMIP6.MOHC.HadGEM3-GC31-LL.amip.none.r1i1p1f3",
    "grid": "N96",
    "grid_label": "gn",
    "initialization_index": "1",
    "institution": "Met Office Hadley Centre, Fitzroy Road, Exeter, Devon, EX1 3PB, UK",
    "institution_id": "MOHC",
    "license": "CMIP6 model data produced by the Met Office Hadley Centre is licensed under a Creative Commons Attribution-ShareAlike 4.0 International License (https://creativecommons.org/licenses). Consult https://pcmdi.llnl.gov/CMIP6/TermsOfUse for terms of use governing CMIP6 output, including citation requirements and proper acknowledgment. Further information about this data, including some limitations, can be found via the further_info_url (recorded as a global attribute in this file) and at https://ukesm.ac.uk/cmip6. The data producers and data providers make no warranty, either express or implied, including, but not limited to, warranties of merchantability and fitness for a particular purpose. All liabilities arising from the supply of the information (including any liability arising in negligence) are excluded to the fullest extent permitted by law.",
    "mip_era": "CMIP6",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "parent_activity_id": "no parent",
    "physics_index": "1",
    "realization_index": "1",
    "source": "HadGEM3-GC31-LL (2016): \naerosol: UKCA-GLOMAP-mode\natmos: MetUM-HadGEM3-GA7.1 (N96; 192 x 144 longitude/latitude; 85 levels; top level 85 km)\natmosChem: none\nland: JULES-HadGEM3-GL7.1\nlandIce: none\nocean: NEMO-HadGEM3-GO6.0 (eORCA1 tripolar primarily 1 deg with meridional refinement down to 1/3 degree in the tropics; 360 x 330 longitude/latitude; 75 levels; top grid cell 0-1 m)\nocnBgchem: none\nseaIce: CICE-HadGEM3-GSI8 (eORCA1 tripolar primarily 1 deg; 360 x 330 longitude/latitude)",
    "source_id": "HadGEM3-GC31-LL",
    "source_type": "AGCM",
    "sub_experiment": "none",
    "sub_experiment_id": "none",
    "tracking_prefix": "hdl:21.14100",
    "variant_label": "r1i1p1f3"
}


class TestHasForecastCoordinates(unittest.TestCase):
    def setUp(self):
        """
        Write out a simple file using CMOR
        """
        # Set up CMOR
        cmor.setup(inpath="TestTables", netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile="cmor.log", create_subdirectories=0)

        # Define dataset using DATASET_INFO
        with open("Test/input_leadtime.json", "w") as input_file_handle:
            json.dump(DATASET_INFO, input_file_handle, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_leadtime.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")


    def test_has_forcast_coordinates(self):
        # load MIP table
        mip_table = "CMIP6_Amon_leadtime.json"
        table_id = cmor.load_table(mip_table)

        # construct axes
        time = cmor.axis(table_entry="time", units="days since 2000-01-01",
                         coord_vals=np.array([30.0, 60.0]),
                         cell_bounds=np.array([15.0, 45.0, 75.0]))
        reftime = cmor.axis(table_entry="reftime1", units="days since 2000-01-01",
                             coord_vals=np.array([10.0]))
        height2m = cmor.axis(table_entry="height2m", units="m", coord_vals=np.array((2.0,)))
        latitude = cmor.axis(table_entry="latitude", units="degrees_north",
                             coord_vals=np.array(range(5)),
                             cell_bounds=np.array(range(6)))
        longitude = cmor.axis(table_entry="longitude", units="degrees_east",
                              coord_vals=np.array(range(5)),
                              cell_bounds=np.array(range(6)))
        axis_ids = [longitude, latitude, time, reftime, height2m]
        tas_var_id = cmor.variable(table_entry="tas", axis_ids=axis_ids, units="K")
        data = np.random.random(50)
        reshaped_data = data.reshape((5, 5, 2, 1, 1))
        # write data
        cmor.write(tas_var_id, reshaped_data)
        filename = cmor.close(tas_var_id, file_name=True)

        ds = Dataset(filename)
        np.testing.assert_array_equal(ds.variables['reftime'][:].data, [10.0])
        np.testing.assert_array_equal(ds.variables['leadtime'][:].data, [20.0, 50.0])
        ds.close()
        os.remove(filename)


class TestWriteSlices(unittest.TestCase):

    def setUp(self):
        """
        Write out a simple file using CMOR
        """
        # Set up CMOR
        cmor.setup(inpath="TestTables", netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile="cmor.log", create_subdirectories=0)

        # Define dataset using DATASET_INFO
        with open("Test/input_leadtime.json", "w") as input_file_handle:
            json.dump(DATASET_INFO, input_file_handle, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json("Test/input_leadtime.json")
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")


    def test_multiple_writes(self):
        # load MIP table
        mip_table = "CMIP6_Amon.json"
        table_id = cmor.load_table(mip_table)

        # construct axes
        time = cmor.axis(table_entry="time", units="days since 2000-01-01")
        height2m = cmor.axis(table_entry="height2m", units="m", coord_vals=np.array((2.0,)))
        latitude = cmor.axis(table_entry="latitude", units="degrees_north",
                             coord_vals=np.array(range(5)),
                             cell_bounds=np.array(range(6)))
        longitude = cmor.axis(table_entry="longitude", units="degrees_east",
                              coord_vals=np.array(range(5)),
                              cell_bounds=np.array(range(6)))
        axis_ids = [longitude, latitude, time, height2m]
        tas_var_id = cmor.variable(table_entry="tas", axis_ids=axis_ids, units="K")
        # write data
        for year in range(2):
            data = np.random.random(25*12)
            reshaped_data = data.reshape((5, 5, 12, 1))
            time_vals = [pt*30 + 360.0 * year + 15.0 for pt in range(12)]
            time_bnds = [360.0 * year ] + [pt*30 + 360.0 * year + 30.0 for pt in range(12)]
            cmor.write(tas_var_id, reshaped_data, time_vals=time_vals, time_bnds=time_bnds)

        filename = cmor.close(tas_var_id, file_name=True)
        ds = Dataset(filename)
        np.testing.assert_array_equal(ds.variables['time'][:].data,  [15.,  45.,  75., 105., 135., 165., 195., 225., 255., 285., 315., 345., 375., 405., 435., 465., 495., 525., 555., 585., 615., 645., 675., 705.])
        ds.close()
        os.remove(filename)


    def test_multiple_writes_with_leadtime(self):
        # load MIP table
        mip_table = "CMIP6_Amon_leadtime.json"
        table_id = cmor.load_table(mip_table)

        # construct axes
        time = cmor.axis(table_entry="time", units="days since 2000-01-01")
        height2m = cmor.axis(table_entry="height2m", units="m", coord_vals=np.array((2.0,)))
        latitude = cmor.axis(table_entry="latitude", units="degrees_north",
                             coord_vals=np.array(range(5)),
                             cell_bounds=np.array(range(6)))
        longitude = cmor.axis(table_entry="longitude", units="degrees_east",
                              coord_vals=np.array(range(5)),
                              cell_bounds=np.array(range(6)))
        reftime = cmor.axis(table_entry="reftime1", units="days since 2000-01-01",
                            coord_vals=np.array([10.0]))
        axis_ids = [longitude, latitude, time, reftime, height2m]

        tas_var_id = cmor.variable(table_entry="tas", axis_ids=axis_ids, units="K")
        # write data
        for year in range(2):
            data = np.random.random(25 * 12)
            reshaped_data = data.reshape((5, 5, 12, 1, 1))
            time_vals = [pt * 30 + 360.0 * year + 15.0 for pt in range(12)]
            time_bnds = [360.0 * year] + [pt * 30 + 360.0 * year + 30.0 for pt in range(12)]
            cmor.write(tas_var_id, reshaped_data, time_vals=time_vals, time_bnds=time_bnds)

        filename = cmor.close(tas_var_id, file_name=True)
        ds = Dataset(filename)
        np.testing.assert_array_equal(ds.variables['time'][:].data,
                                      [15., 45., 75., 105., 135., 165., 195., 225., 255., 285., 315., 345., 375., 405.,
                                       435., 465., 495., 525., 555., 585., 615., 645., 675., 705.])
        ds.close()
        os.remove(filename)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import cmor
import numpy


def main():

    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_4)
    cmor.dataset_json("Test/CMOR_input_example.json")

    table = 'CMIP6_Amon.json'
    cmor.load_table(table)
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'plev19',
             'units': 'Pa',
             'coord_vals': [100000., 92500., 85000., 70000., 60000., 50000., 40000., 30000., 25000., 20000., 15000., 10000., 7000., 5000., 3000., 2000., 1000., 500., 100.]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    axis_ids = list()
    for axis in axes:
        print('doing:', axis)
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    for var, units, value in (('ta', 'K', 274), ('ua', 'm s-1', 10)):
        values = numpy.array([value, ] *
                             len(axes[1]['coord_vals']), numpy.float32)
        varid = cmor.variable(var,
                              units,
                              axis_ids,
                              history='variable history',
                              missing_value=-99
                              )
        cmor.set_variable_attribute(varid, 'cell_measures', 'BLABLABLA')
        cmor.write(varid, values, time_vals=[15], time_bnds=[[0, 30]])

    cmor.close()


if __name__ == '__main__':

    main()



'''Tests around pywrapper - need an environment to pick up the stub _cmor implementation'''
import numpy

import unittest
from cmor import grid

import _cmor_stub  # this should be a stub cmor implementation


class TestGrid(unittest.TestCase):

    def test_pass_only_axis_ids(self):
        for axis_ids in ([1, 2], numpy.array([1, 2])):
            args = grid(axis_ids)
            self.assert_on_axis_id_args(axis_ids, args)
            self.assert_on_default_coordinates(args)
            self.assert_on_default_vertices(args)

    def test_lat_lon_grid(self):
        axis_ids = [1, 2]
        for atype in (numpy.array, list, tuple):
            lats = atype(numpy.arange(2 * 3).reshape(2, 3) + 0.1)
            lons = atype(numpy.arange(2 * 3).reshape(2, 3) - 0.1)
            args = grid(axis_ids, latitude=lats, longitude=lons)
            self.assert_on_axis_id_args(axis_ids, args)
            self.assert_on_coordiantes(lats, lons, args)
            self.assert_on_default_vertices(args)

    def test_lat_lon_with_vertices(self):
        axis_ids = [1, 2]
        nvert = 4
        for atype in (numpy.array, list, tuple):
            lats = atype(numpy.arange(2 * 3).reshape(2, 3) + 0.1)
            lons = atype(numpy.arange(2 * 3).reshape(2, 3) - 0.1)
            lat_vert = atype(
                numpy.arange(
                    2 *
                    3 *
                    nvert).reshape(
                    2,
                    3,
                    nvert) +
                0.5)
            lon_vert = atype(
                numpy.arange(
                    2 *
                    3 *
                    nvert).reshape(
                    2,
                    3,
                    nvert) -
                0.5)
            args = grid(axis_ids,
                        latitude=lats,
                        longitude=lons,
                        latitude_vertices=lat_vert,
                        longitude_vertices=lon_vert)
            self.assert_on_axis_id_args(axis_ids, args)
            self.assert_on_coordiantes(lats, lons, args)
            self.assert_on_vertices(nvert, lat_vert, lon_vert, args)

    def assert_on_axis_id_args(self, axis_ids, args):
        self.assertEquals(len(axis_ids), args[0])
        self.assertTrue(((axis_ids) == args[1]).all())

    def assert_on_coordiantes(self, lats, lons, args):
        self.assertEquals('d', args[2])
        self.assertTrue((lats == args[3]).all())
        self.assertTrue((lons == args[4]).all())

    def assert_on_vertices(self, nvert, lat_vert, lon_vert, args):
        self.assertEquals(nvert, args[5])
        self.assertTrue((lat_vert == args[6]).all())
        self.assertTrue((lon_vert == args[7]).all())

    def assert_on_default_coordinates(self, args):
        self.assertEquals('f', args[2])
        self.assertEquals(None, args[3])
        self.assertEquals(None, args[4])

    def assert_on_default_vertices(self, args):
        self.assertEquals(0, args[5])
        self.assertEquals(None, args[6])
        self.assertEquals(None, args[7])


class TestGridCallErrors(unittest.TestCase):
    def test_error_rank_axis_ids(self):
        try:
            grid([[1], [2]])
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals('error axes list/array must be 1D', str(e))

    def test_error_on_axis_ids(self):
        bad_axis_ids = (0, 'astring')
        for axis_ids in bad_axis_ids:
            try:
                grid(axis_ids)
                self.fail('should raise exception')
            except Exception as e:
                self.assertEquals('Error could not convert axis_ids list to a numpy array',
                                  str(e))

    def test_error_latitude_no_longitude(self):
        try:
            grid([0], latitude=numpy.arange(2))
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals(
                "Error could not convert longitude to a numpy array", str(e))

    def test_error_longitude_no_latitude(self):
        try:
            grid([0], longitude=numpy.arange(2))
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals(
                "latitude and longitude must be BOTH an array or None", str(e))

    def test_error_type_lats(self):
        lons = numpy.arange(2 * 3).reshape(2, 3)
        for lats in (0, 0.1, 'string', {}):
            try:
                grid([0, 11], latitude=lats, longitude=lons)
                self.fail('should raise exception')
            except Exception as e:
                self.assertEquals(
                    'Error could not convert latitude to a numpy array', str(e))

    def test_error_type_lons(self):
        lats = numpy.arange(2 * 3).reshape(2, 3)
        for lons in (0, 0.1, 'string', {}):
            try:
                grid([0, 1], latitude=lats, longitude=lons)
                self.fail('should raise exception')
            except Exception as e:
                self.assertEquals(
                    'Error could not convert longitude to a numpy array', str(e))

    def test_error_rank_lons(self):
        axis_ids = [1, 2]
        lats = numpy.arange(2 * 3).reshape(2, 3)
        lons = numpy.arange(3)
        try:
            grid(axis_ids, latitude=lats, longitude=lons)
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals(
                "longitude's rank does not match number of axes passed via axis_ids", str(e))

    def test_error_rank_lats(self):
        axis_ids = [1, 2]
        lats = numpy.arange(2)
        lons = numpy.arange(2 * 3).reshape(2, 3)
        try:
            grid(axis_ids, latitude=lats, longitude=lons)
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals(
                "latitude's rank does not match number of axes passed via axis_ids", str(e))

    def test_error_rank_lat_verts(self):  # this test may be fragile?
        axis_ids = [1, 2]
        lats = numpy.arange(2 * 3).reshape(2, 3)
        lons = numpy.arange(2 * 3).reshape(2, 3)
        lat_verts = lats
        try:
            grid(axis_ids, latitude=lats, longitude=lons,
                 latitude_vertices=lat_verts)
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals("latitude_vertices's rank does not match number of axes passed via axis_ids +1 (for vertices)",
                              str(e))

    def test_error_rank_lon_verts(self):  # this test may be fragile?
        axis_ids = [1, 2]
        lats = numpy.arange(2 * 3).reshape(2, 3)
        lons = numpy.arange(2 * 3).reshape(2, 3)
        lon_verts = lons
        try:
            grid(axis_ids, latitude=lats, longitude=lons,
                 longitude_vertices=lon_verts)
            self.fail('should raise exception')
        except Exception as e:
            self.assertEquals("longitude_vertices's rank does not match number of axes passed via axis_ids +1 (for vertices)",
                              str(e))


if __name__ == '__main__':
    unittest.main()



from out_files import out
from in_files import input_tables
import sys
import os

import cmor

test = sys.argv[1]

test = os.path.split(test)[1]
if test[-4:].lower() == '.f90':
    test = test[:-4]

print('Checking results for:', test)

outfiles = out.get(test, [])
intables = input_tables.get(test, ['IPCC_test_table_A', ])


class CMORResultCheckError(Exception):
    def __init__(self, args=None):
        self.args = args


nfiles = 0

print('files:', outfiles)
gotfiles = []
missing = []
for f in outfiles:
    if f is None:
        print('No checking')
        sys.exit()
    tables = []
    for t in intables:
        tables.append(os.path.join("Test", t))
    tbl = tables.pop(0)
    if len(tables) == 0:
        tables = [None, ]
    fnm = os.path.join("Test", f)
    if os.path.exists(fnm):
        nfiles += 1
        gotfiles.append(fnm)
        print('Checking output file:', f)
        cmor.checkCMOR(sys.stdout, fnm, tbl, other_tables=tables)
        print('----------------------- Success ------------------------------')
       # os.remove(fnm)
    else:
        missing.append(fnm)
if nfiles == 0 and outfiles != []:
    raise CMORResultCheckError(
        ["Error could not find any output file for test: Test/%s.f90" % (test), ])
elif nfiles != len(outfiles):
    raise CMORResultCheckError(
        [
            "Error checking output files for test: Test/%s.f90 we could only find %i files when %i were expected.\n\n Expected files: \n\t%s\n\nPresent files: \n\t%s\n\nMissing files: \n\t%s\n" %
            (test,
             nfiles,
             len(outfiles),
             repr(outfiles),
             repr(gotfiles),
             repr(missing)),
        ])



import cmor

cmor.setup(
	# inpath has to point to the CMOR 
	# tables path (CMIP6, input4MIPs or otherwise)
	inpath='Tables',  
	netcdf_file_action=cmor.CMOR_REPLACE_4
)

cmor.dataset_json("Test/CMOR_input_example.json")    
  
# Loading this test table overwrites the normal CF checks on valid variable values.
# This is perfect for testing but shouldn't be done when writing real data.
table='CMIP6_Amon.json'
cmor.load_table(table)


# here is where you add your axes
itime = cmor.axis(table_entry= 'time',
                  units= 'days since 2000-01-01 00:00:00',
                  coord_vals= [15,],
                  cell_bounds= [0, 30])
ilat = cmor.axis(table_entry= 'latitude',
                 units= 'degrees_north',
                 coord_vals= [0],
                 cell_bounds= [-1, 1])
ilon = cmor.axis(table_entry= 'longitude',
                 units= 'degrees_east',
                 coord_vals= [90],
                 cell_bounds= [89, 91])

axis_ids = [itime,ilat,ilon]
              
# here we create a variable with appropriate name, units and axes
varid = cmor.variable('ts', 'K', axis_ids)

# then we can write the variable along with the data
cmor.write(varid, [273])

# finally we close the file and print where it was saved
outfile = cmor.close(varid, file_name=True)
print("File written to: {}".format(outfile))
cmor.close()



import cmor
import numpy


def cmor_initialisation():
    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               create_subdirectories=0)
    cmor.dataset_json("Test/CMOR_input_example.json")


def setup_data():
    axes = [{'table_entry': 'time1',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'site',
             'units': '',
             'coord_vals': [0]},
            ]

    values = numpy.array([215.], numpy.float32)
    return values, axes


def cmor_define_and_write(values, axes):
    table = 'CMIP6_CFsubhr.json'
    tid1 = cmor.load_table(table)
    site_axis_id = cmor.axis(**axes[1])

    time_axis_id = cmor.axis(**axes[0])

    table2 = 'CMIP6_grids.json'
    tid2 = cmor.load_table(table2)
    gid = cmor.grid([site_axis_id, ], latitude=numpy.array(
        [-20, ]), longitude=numpy.array([150, ]))

    axis_ids = [time_axis_id, gid]
    cmor.set_table(tid1)
    varid = cmor.variable('rlut',
                          'W m-2',
                          axis_ids,
                          history='variable history',
                          missing_value=-99,
                          positive='up'
                          )

    cmor.write(varid, values, time_vals=[15])


def main():

    cmor_initialisation()
    values, axes = setup_data()
    cmor_define_and_write(values, axes)
    print(cmor.close(file_name=True))


if __name__ == '__main__':

    main()



import cmor


def multi_call_test():
    cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)

    cmor.dataset_json("Test/CMOR_input_example.json")
    table = 'CMIP6_Amon.json'
    cmor.load_table(table)
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)
    varid = cmor.variable('ts', 'K', axis_ids)
    cmor.write(varid, [275], time_vals=[15], time_bnds=[[0, 30]])
    print('First write worked as expected')
    try:
        cmor.write(varid, [275], time_vals=[15], time_bnds=[[0], [30]])
        raise Exception("We shouldn't be getting in here")
    except BaseException:
        print('Second write that should have failed did fail, good!')
        pass
    cmor.close(varid)
    print('Success')


if __name__ == '__main__':
    multi_call_test()



import json
import cmor
import unittest
import os
import numpy
import base_CMIP6_CV

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"

USER_INPUT = {
    "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
    "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
    "_cmip7_option": 1,
    "_controlled_vocabulary_file": CV_PATH,
    "activity_id": "CMIP",
    "branch_time_in_child": 30.0,
    "branch_time_in_parent": 10800.0,
    "calendar": "360_day",
    "cv_version": "6.2.19.0",
    "drs_specs": "MIP-DRS7",
    "experiment_id": "piControl",
    "forcing_index": "f30",
    "grid_label": "gn",
    "initialization_index": "i000001d",
    "institution_id": "PCMDI",
    "license_id": "CC BY 4.0",
    "nominal_resolution": "250 km",
    "outpath": ".",
    "parent_mip_era": "CMIP7",
    "parent_time_units": "days since 1850-01-01",
    "parent_activity_id": "CMIP",
    "parent_source_id": "PCMDI-test-1-0",
    "parent_experiment_id": "piControl",
    "parent_variant_label": "r1i1p1f3",
    "physics_index": "p1",
    "realization_index": "r009",
    "source_id": "PCMDI-test-1-0",
    "tracking_prefix": "hdl:21.14100",
    "host_collection": "CMIP7",
    "frequency": "mon",
    "region": "glb",
    "archive_id": "WCRP"
}


class TestTimeBoundsMismatchMakingMonthlyAxis(base_CMIP6_CV.BaseCVsTest):

    def setUp(self):
        super().setUp()

        frequency = "mon"
        self.input_path = f"Test/input_making_{frequency}_axis.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(self.input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["frequency"] = frequency
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json(self.input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")
        
        cmor.load_table("CMIP7_ocean.json")

    def tearDown(self):
        os.remove(self.input_path)
        return super().tearDown()
        
    def assertWarningMessage(self, flag):
        warn_msg = (
            "Warning: The values you provided for axis time are different "
            "from those computed from the bounds, which are used for the axis "
            "values instead of the user-provided values."
        )

        with open(self.tmpfile, 'r') as f:
            lines = f.readlines()
            warning_found = any(warn_msg in l for l in lines)
            self.assertEqual(flag, warning_found)

    def test_make_axis_monthly_with_day_units_no_warning(self):
        
        time = numpy.array([15.5, 45.5])
        time_bnds = numpy.array([0, 31, 60])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="days since 2018")

        self.assertWarningMessage(False)

    def test_make_axis_monthly_with_day_units_with_warning(self):
        
        time = numpy.array([15.5, 45])
        time_bnds = numpy.array([0, 31, 60])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="days since 2018")

        self.assertWarningMessage(True)
        
        self.assertCV("45.000000 will be replaced with 45.500000 "
                      "between bounds 31.000000 and 60.000000", 
                      "! The first value found is at index 1: ")
        

    def test_make_axis_monthly_with_month_units(self):
        
        time = numpy.array([0, 1])
        time_bnds = numpy.array([0, 1, 2])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="months since 2018")

        self.assertWarningMessage(True)
        
        self.assertCV("0.000000 will be replaced with 15.000000 "
                      "between bounds 0.000000 and 30.000000", 
                      "! The first value found is at index 0: ")


class TestTimeBoundsMismatchMaking6HourlyAxis(base_CMIP6_CV.BaseCVsTest):

    def setUp(self):
        super().setUp()

        frequency = "6hr"
        self.input_path = f"Test/input_making_{frequency}_axis.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(self.input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["frequency"] = frequency
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json(self.input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")
        
        cmor.load_table("CMIP7_ocean.json")

    def tearDown(self):
        os.remove(self.input_path)
        return super().tearDown()
        
    def assertWarningMessage(self, flag):
        warn_msg = (
            "Warning: The values you provided for axis time are different "
            "from those computed from the bounds, which are used for the axis "
            "values instead of the user-provided values."
        )

        with open(self.tmpfile, 'r') as f:
            lines = f.readlines()
            warning_found = any(warn_msg in l for l in lines)
            self.assertEqual(flag, warning_found)

    def test_make_axis_6hourly_with_day_units_no_warning(self):
        
        time = numpy.array([0.125, 0.375])
        time_bnds = numpy.array([0, 0.25, 0.5])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="days since 2018")

        self.assertWarningMessage(False)

    def test_make_axis_6hourly_with_day_units_with_warning(self):
        
        time = numpy.array([0, 0.25])
        time_bnds = numpy.array([0, 0.25, 0.5])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="days since 2018")

        self.assertWarningMessage(True)
        
        self.assertCV("0.000000 will be replaced with 0.125000 "
                      "between bounds 0.000000 and 0.250000", 
                      "! The first value found is at index 0: ")


class TestTimeBoundsMismatchMakingYearlyAxis(base_CMIP6_CV.BaseCVsTest):

    def setUp(self):
        super().setUp()

        frequency = "yr"
        self.input_path = f"Test/input_making_{frequency}_axis.json"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(self.input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["frequency"] = frequency
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json(self.input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")
        
        cmor.load_table("CMIP7_ocean.json")

    def tearDown(self):
        os.remove(self.input_path)
        return super().tearDown()
        
    def assertWarningMessage(self, flag):
        warn_msg = (
            "Warning: The values you provided for axis time are different "
            "from those computed from the bounds, which are used for the axis "
            "values instead of the user-provided values."
        )

        with open(self.tmpfile, 'r') as f:
            lines = f.readlines()
            warning_found = any(warn_msg in l for l in lines)
            self.assertEqual(flag, warning_found)

    def test_make_axis_yearly_with_day_units_no_warning(self):
        
        time = numpy.array([182.5])
        time_bnds = numpy.array([0, 365])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="days since 2018")

        self.assertWarningMessage(False)

    def test_make_axis_yearly_with_day_units_with_warning(self):
        
        time = numpy.array([0])
        time_bnds = numpy.array([0, 365])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="days since 2018")

        self.assertWarningMessage(True)
        
        self.assertCV("0.000000 will be replaced with 182.500000 "
                      "between bounds 0.000000 and 365.000000", 
                      "! The first value found is at index 0: ")

    def test_make_axis_yearly_with_year_units_no_warning(self):
        
        time = numpy.array([0.5])
        time_bnds = numpy.array([0, 1])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="years since 2018")

        self.assertWarningMessage(False)

    def test_make_axis_yearly_with_year_units_with_warning(self):
        
        time = numpy.array([0.5, 1])
        time_bnds = numpy.array([0, 1, 2])

        _ = cmor.axis("time",
                      coord_vals=time,
                      cell_bounds=time_bnds,
                      units="years since 2018")

        self.assertWarningMessage(True)
        
        self.assertCV("360.000000 will be replaced with 540.000000 "
                      "between bounds 360.000000 and 720.000000", 
                      "! The first value found is at index 1: ")


class TestTimeBoundsMismatchTimesPassed(base_CMIP6_CV.BaseCVsTest):
        
    def assertWarningMessage(self, flag):
        warn_msg = (
            "Warning: The values you provided for axis time are different "
            "from those computed from the bounds, which are used for the axis "
            "values instead of the user-provided values."
        )

        with open(self.tmpfile, 'r') as f:
            lines = f.readlines()
            warning_found = any(warn_msg in l for l in lines)
            self.assertEqual(flag, warning_found)

    def test_time_value_bounds_mismatch_times_passed(self):

        test_name = "time_value_bounds_mismatch_times_passed"
        input_path = f"Test/input_{test_name}.json"
        frequency = "mon"

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE,
                   logfile=self.tmpfile)

        with open(input_path, "w") as input_file:
            user_input = USER_INPUT.copy()
            user_input["frequency"] = frequency
            json.dump(user_input, input_file, sort_keys=True, indent=4)

        # read dataset info
        error_flag = cmor.dataset_json(input_path)
        if error_flag:
            raise RuntimeError("CMOR dataset_json call failed")

        lat = numpy.array([10, 20, 30])
        lat_bnds = numpy.array([5, 15, 25, 35])
        lon = numpy.array([0, 90, 180, 270])
        lon_bnds = numpy.array([-45, 45, 135, 225, 315])
        time = numpy.array([15.5, 45])
        time_bnds = numpy.array([0, 31, 60])
        tos_shape = (time.shape[0], lat.shape[0], lon.shape[0])
        tos = numpy.full(tos_shape, 27)
        cmor.load_table("CMIP7_ocean.json")
        cmorlat = cmor.axis("latitude",
                            coord_vals=lat,
                            cell_bounds=lat_bnds,
                            units="degrees_north")
        cmorlon = cmor.axis("longitude",
                            coord_vals=lon,
                            cell_bounds=lon_bnds,
                            units="degrees_east")
        cmortime = cmor.axis("time",
                             units="days since 2018")
        axes = [cmortime, cmorlat, cmorlon]
        cmortos = cmor.variable("tos_tavg-u-hxy-sea", "degC", axes)

        self.assertWarningMessage(False)

        self.assertEqual(cmor.write(cmortos, tos, ntimes_passed=2,
                                    time_vals=time, time_bnds=time_bnds), 0)

        self.assertWarningMessage(True)
        
        self.assertCV("45.000000 will be replaced with 45.500000 "
                      "between bounds 31.000000 and 60.000000", 
                      "! The first value found is at index 1: ")

        os.remove(input_path)


if __name__ == '__main__':
    unittest.main()



import cmor
import numpy
import unittest
from base_CMIP6_CV import BaseCVsTest


def run():
    unittest.main()


class TestNaNCheck(BaseCVsTest):

    def test_nan_check(self):
        cmor.setup(inpath='Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.load_table("CMIP6_Amon.json")

        itime = cmor.axis(table_entry='time',
                          units='days since 2000-01-01 00:00:00',
                          coord_vals=[15, 45, 75],
                          cell_bounds=[0, 30, 60, 90])
        ilat = cmor.axis(table_entry='latitude',
                         units='degrees_north',
                         coord_vals=[0],
                         cell_bounds=[-1, 1])
        ilon = cmor.axis(table_entry='longitude',
                         units='degrees_east',
                         coord_vals=[90],
                         cell_bounds=[89, 91])

        axis_ids = [itime, ilat, ilon]

        varid = cmor.variable('ts', 'K', axis_ids)

        with self.assertRaises(cmor.CMORError):
            _ = cmor.write(varid, [273, numpy.nan, 273])

        self.assertCV(
            "Invalid value(s) detected for variable 'ts' "
            "(table: Amon): 1 values were NaNs."
        )
        self.assertCV(
            "time: 1/45 lat: 0/0 lon: 0/90",
            line_trigger="First encountered NaN was at (axis: index/value):"
        )

    def test_nan_check_with_basin_axis(self):
        cmor.setup(inpath='Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.load_table("CMIP6_Omon.json")

        ntime = 12
        nlat = 90
        dlat = 180 / nlat
        lats = numpy.arange(-90 + dlat / 2., 90, dlat)
        blats = numpy.arange(-90, 90 + dlat, dlat)

        ilat = cmor.axis(
            table_entry='latitude',
            coord_vals=lats,
            cell_bounds=blats,
            units='degrees_north')

        myregions = [
            "atlantic_ocean",
            "indian_pacific_ocean",
            "pacific_ocean",
            "atlantic_arctic_ocean",
            "global_ocean"]
        nreg = len(myregions)
        ireg = cmor.axis(table_entry='basin', units='1', coord_vals=myregions)

        itim = cmor.axis(
            table_entry='time',
            units='months since 2030',
            interval='1 month',
            coord_vals=numpy.arange(ntime),
            cell_bounds=numpy.arange(
                ntime + 1))

        varid = cmor.variable(table_entry='htovgyre', units='W',
                              axis_ids=numpy.array([itim, ireg, ilat]))

        data = numpy.random.random((ntime, nreg, nlat)) * 3.E14
        data[3, 2, 5] = numpy.nan

        with self.assertRaises(cmor.CMORError):
            _ = cmor.write(varid, data)

        self.assertCV(
            "Invalid value(s) detected for variable 'htovgyre' "
            "(table: Omon): 1 values were NaNs."
        )
        self.assertCV(
            "time: 3/105 basin: 2/pacific_ocean lat: 5/-79",
            line_trigger="First encountered NaN was at (axis: index/value):"
        )


if __name__ == '__main__':
    run()



#!/usr/bin/env python
import cmor
import numpy


def main():

    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE)
    cmor.dataset_json("Test/CMOR_input_example.json")

    table = 'CMIP6_6hrLev.json'
    cmor.load_table(table)
    axes = [{'table_entry': 'time1',
             'units': 'days since 2000-01-01 00:00:00'
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            {'table_entry': 'hybrid_height',
             'coord_vals': [0, 1],
             'cell_bounds': [[0., 0.5], [0.5, 1.]],
             'units': 'm',
             },
            ]

    values = numpy.array([1.2, 1.2], numpy.float32)
    numpy.reshape(values, (2, 1, 1, 1))
    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    print('cmor.axis calls complete')

    cmor.zfactor(axis_ids[3], 'b', '', axis_ids[3:4],
                 'd', [0., 0.5], [[0., 0.25], [0.25, 1.]])
    cmor.zfactor(axis_ids[3], 'orog', 'm', axis_ids[1:3], 'd', [[0.]])
    print('cmor.zfactor calls complete')
    varid = cmor.variable('ua',
                          'm s-1',
                          axis_ids,
                          missing_value=-99
                          )

    print('cmor.variable call complete')

    cmor.write(varid, values, time_vals=[6.0], time_bnds=[3., 12.])

    print('cmor.write call complete')

    cmor.close()


if __name__ == '__main__':

    main()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/' and 'Test/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/
#      test_python_CMIP6_experimentID.json to the 'Test/' directory.
#
# pylint: disable = missing-docstring, invalid-name
"""
Tests for ``cmor.has_variable_attribute``.
"""
import numpy as np
import os
import unittest

import cmor


class TestHasVariableAttribute(unittest.TestCase):
    """
    Tests for ``cmor.has_variable_attribute``.
    """

    def setUp(self):
        self.logfile = 'has_variable_attribute.log'
        cmor.setup(inpath='Tables', logfile=self.logfile)
        cmor.dataset_json('Test/test_python_CMIP6_experimentID.json')
        cmor.load_table('CMIP6_Omon.json')
        coord_vals = np.array([0., 1., 2., 3., 4.])
        cell_bounds = np.array([0., 1., 2., 3., 4., 5.])
        axis_id = cmor.axis(table_entry='time', units='months since 2010',
                            coord_vals=coord_vals, cell_bounds=cell_bounds)
        self.variable_id = cmor.variable(table_entry='masso',
                                         axis_ids=[axis_id], units='kg')
        cmor.set_variable_attribute(self.variable_id, 'valid_attribute','c', 'valid_value')

    def test_has_variable_attribute_with_valid_attribute(self):
        self.assertTrue(cmor.has_variable_attribute(self.variable_id,
                                                    'valid_attribute'))

    # Test an attribute that does not exist
    def test_has_variable_attribute_with_invalid_attribute(self):
        self.assertFalse(cmor.has_variable_attribute(self.variable_id,
                                                     'invalid_attribute'))

    def test_set_variable_attribute_with_string_type(self):
        cmor.set_variable_attribute(self.variable_id, 'myStringAttr','c',
                                    'valid_value')
        self.assertTrue(cmor.has_variable_attribute(self.variable_id,
                                                     'myStringAttr'))

    def test_set_variable_attribute_with_long_type(self):
        cmor.set_variable_attribute(self.variable_id, 'long_attribute','l',
                                    1234)
        self.assertTrue(cmor.has_variable_attribute(self.variable_id,
                                                     'long_attribute'))

    def test_set_variable_attribute_with_int_type(self):
        cmor.set_variable_attribute(self.variable_id, 'int_attribute','i',
                                    1134)
        self.assertTrue(cmor.has_variable_attribute(self.variable_id,
                                                     'int_attribute'))

    def test_set_variable_attribute_with_right_type(self):
        cmor.set_variable_attribute(self.variable_id, 'float_attribute','f',
                                    1.0)
        self.assertTrue(cmor.has_variable_attribute(self.variable_id,
                                                     'float_attribute'))
    # We get a warning
    def test_set_variable_attribute_with_different_type(self):
        cmor.set_variable_attribute(self.variable_id, 'double_attribute','d',
                                    12.34)
        self.assertTrue(cmor.has_variable_attribute(self.variable_id,
                                                     'double_attribute'))
    def tearDown(self):
        cmor.close(self.variable_id)
        if os.path.isfile(self.logfile):
            os.remove(self.logfile)


if __name__ == '__main__':
    unittest.main()



import cmor
import json
import os
import unittest
from base_CMIP6_CV import BaseCVsTest

CMIP7_TABLES_PATH = "cmip7-cmor-tables/tables"
CV_PATH = "TestTables/CMIP7_CV.json"


def run():
    unittest.main()


class TestFrequencyRequired(BaseCVsTest):
    def _create_cmip7_json(self, frequency=None, extra_fields=None):
        """Helper to create CMIP7-compliant JSON data"""
        json_data = {
            "_AXIS_ENTRY_FILE": "CMIP7_coordinate.json",
            "_FORMULA_VAR_FILE": "CMIP7_formula_terms.json",
            "_cmip7_option": 1,
            "_controlled_vocabulary_file": CV_PATH,
            "mip_era": "CMIP7",
            "institution_id": "PCMDI",
            "source_id": "PCMDI-test-1-0",
            "experiment_id": "piControl",
            "activity_id": "CMIP",
            "source_type": "AOGCM",
            "grid_label": "gn",
            "nominal_resolution": "250 km",
            "calendar": "360_day",
            "region": "glb",
            "outpath": ".",
        }

        if frequency is not None:
            json_data["frequency"] = frequency

        if extra_fields:
            json_data.update(extra_fields)

        return json_data

    def test_missing_frequency_with_time_axis_error(self):
        """Frequency should be required for variables with time axis"""
        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        # Create JSON without frequency
        json_data = self._create_cmip7_json(frequency=None)
        json_file = "test_no_frequency_cmip7.json"
        with open(json_file, 'w') as f:
            json.dump(json_data, f)

        cmor.dataset_json(json_file)
        cmor.load_table("CMIP7_atmos.json")

        itime = cmor.axis(table_entry='time', units='days since 2000-01-01')
        ilat = cmor.axis(table_entry='latitude', units='degrees_north',
                         coord_vals=[0], cell_bounds=[-0.5, 0.5])
        ilon = cmor.axis(table_entry='longitude', units='degrees_east',
                         coord_vals=[90], cell_bounds=[89.5, 90.5])

        with self.assertRaises(cmor.CMORError):
            cmor.variable('prra_tavg-u-hxy-is', 'kg m-2 s-1', [itime, ilat, ilon])

        # Use assertCV to check error message in log
        self.assertCV("3.14.1", "Error:", number_of_lines_to_scan=10)
        self.assertCV("frequency' attribute is required", "Error:", number_of_lines_to_scan=10)
        self.assertCV(json_file, "Error:", number_of_lines_to_scan=10)
        os.remove(json_file)

    def test_fx_field_without_frequency_passes(self):
        """Fixed fields should not require frequency"""
        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        # Create JSON without frequency - should be OK for fx fields
        json_data = self._create_cmip7_json(frequency=None)
        json_file = "test_fx_no_freq_cmip7.json"
        with open(json_file, 'w') as f:
            json.dump(json_data, f)

        cmor.dataset_json(json_file)
        cmor.load_table("CMIP7_land.json")

        ilat = cmor.axis(table_entry='latitude', units='degrees_north',
                         coord_vals=[0], cell_bounds=[-0.5, 0.5])
        ilon = cmor.axis(table_entry='longitude', units='degrees_east',
                         coord_vals=[90], cell_bounds=[89.5, 90.5])

        # Should not raise error for fixed field
        var_id = cmor.variable('rootd_ti-u-hxy-lnd', 'kg m-2 s-1', [ilat, ilon])
        self.assertGreaterEqual(var_id, 0)
        os.remove(json_file)

    def test_frequency_mismatch_cmip6(self):
        """CMIP6 tables should warn about frequency mismatch but use table frequency"""
        cmor.setup(inpath='cmip6-cmor-tables/Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        # Create CMIP6 JSON with wrong frequency - will generate warning
        json_data = {
            "mip_era": "CMIP6",
            "institution_id": "PCMDI",
            "source_id": "PCMDI-test-1-0",
            "experiment_id": "piControl",
            "activity_id": "CMIP",
            "source_type": "AOGCM",
            "grid_label": "gn",
            "nominal_resolution": "250 km",
            "frequency": "day",  # Wrong - Amon table expects 'mon'
            "calendar": "360_day",
            "outpath": ".",
        }
        json_file = "test_freq_mismatch_cmip6.json"
        with open(json_file, 'w') as f:
            json.dump(json_data, f)

        cmor.dataset_json(json_file)
        cmor.load_table("CMIP6_Amon.json")

        itime = cmor.axis(table_entry='time', units='days since 2000-01-01')
        ilat = cmor.axis(table_entry='latitude', units='degrees_north',
                         coord_vals=[0], cell_bounds=[-0.5, 0.5])
        ilon = cmor.axis(table_entry='longitude', units='degrees_east',
                         coord_vals=[90], cell_bounds=[89.5, 90.5])

        # Should succeed (no longer raises error)
        var_id = cmor.variable('tas', 'K', [itime, ilat, ilon])
        self.assertGreaterEqual(var_id, 0)

        # Verify warning message was logged (use "Frequency" as trigger to avoid other warnings)
        self.assertCV("Frequency mismatch", "Frequency", number_of_lines_to_scan=6)
        self.assertCV("day", "Frequency", number_of_lines_to_scan=6)
        self.assertCV("mon", "Frequency", number_of_lines_to_scan=6)
        self.assertCV(json_file, "Frequency", number_of_lines_to_scan=6)
        os.remove(json_file)

    def test_hardcoded_frequency_mappings_work(self):
        """Verify hardcoded mappings work without CV frequency section"""
        test_cv_path = "TestTables/CMIP7_CV_no_frequency.json"

        # Create CV without frequency section
        with open(CV_PATH, "r") as cv_infile:
            cv = json.load(cv_infile)
            cv["CV"].pop("frequency", None)  # Remove frequency section
            with open(test_cv_path, "w") as cv_outfile:
                json.dump(cv, cv_outfile, sort_keys=True, indent=4)

        # Create JSON with 6hr frequency, using modified CV
        json_data = self._create_cmip7_json(frequency="6hr")
        json_data["_controlled_vocabulary_file"] = test_cv_path
        json_file = "test_hardcoded_freq_cmip7.json"
        with open(json_file, 'w') as f:
            json.dump(json_data, f)

        cmor.setup(inpath=CMIP7_TABLES_PATH,
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)
        cmor.dataset_json(json_file)

        # Load table that should work with 6hr frequency
        cmor.load_table("CMIP7_atmos.json")

        # Should use hardcoded 0.25 day (6 hour) interval
        time_axis = cmor.axis(table_entry='time',
                              units='hours since 2000-01-01',
                              coord_vals=[3, 9, 15, 21, 27],
                              cell_bounds=[0, 6, 12, 18, 24, 30])

        self.assertGreaterEqual(time_axis, 0)

        # Clean up temporary files
        os.remove(json_file)
        os.remove(test_cv_path)


if __name__ == '__main__':
    run()



import numpy
import cmor
from time import localtime, strftime
today = strftime("%Y%m%d", localtime())

print('Done importing')
try:
    import cdms2
except BaseException:
    print("This test code needs cdms2 interface for i/0")
    import sys
    sys.exit()
import os

dpth = "data"
myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')


def read_input(var, order=None):
    f = cdms2.open(os.path.join(dpth, "%s_sample.nc" % var))
    ok = f(var)
    if order is None:
        s = f(var)
    else:
        s = f(var, order=order)
    s.units = f[var].units
    s.id = var
    f.close()
    return s, ok


def prep_var(data):
    rk = data.rank()
    axes = []
    for i in range(rk):
        ax = data.getAxis(i)
        if ax.isLongitude():
            id = cmor.axis(
                table_entry='longitude',
                units=ax.units,
                coord_vals=ax[:],
                cell_bounds=ax.getBounds())
        elif ax.isLatitude():
            id = cmor.axis(
                table_entry='latitude',
                units=ax.units,
                coord_vals=ax[:],
                cell_bounds=ax.getBounds())
        else:
            id = cmor.axis(
                table_entry=str(
                    ax.id),
                units=ax.units,
                coord_vals=ax[:],
                cell_bounds=ax.getBounds())
            print(i, 'units:', ax.units, ax[0])
        axes.append(id)
    var = cmor.variable(table_entry=data.id,
                        units=data.units,
                        axis_ids=numpy.array(axes),
                        missing_value=data.missing_value,
                        history="rewrote by cmor via python script")
    return var


def prep_cmor():
    cmor.setup(
        inpath="Tables",
        set_verbosity=cmor.CMOR_QUIET,
        netcdf_file_action=cmor.CMOR_REPLACE,
        exit_control=cmor.CMOR_EXIT_ON_MAJOR)
    cmor.dataset_json("Test/CMOR_input_example.json")

    tables = []
    a = cmor.load_table("CMIP6_Omon.json")
    tables.append(a)
    tables.append(cmor.load_table("CMIP6_Amon.json"))
    return


for var in ['tas', ]:
    print('Testing var:', var)
    orders = ['tyx...', 'txy...', 'ytx...', 'yxt...', 'xyt...', 'xty...', ]
    for o in orders:
        print('\tordering:', o)
        data, data_ordered = read_input(var, order=o)
        prep_cmor()
        print(data.shape)
        var_id = prep_var(data)
        df = data.filled(data.missing_value)
        cmor.write(var_id, df)
        cmor.close()
#        fn = "CMIP6/CMIP/CSIRO-BOM/NICAM/piControl/r1i1p1f1/Amon/%s/gn/v%s/%s_Amon_piControl_NICAM_r1i1p1f1_gn_197901-199605.nc" %(var,today,var)
        fn = "CMIP6/CMIP6/ISMIP6/PCMDI/PCMDI-test-1-0/piControl-withism/r3i1p1f1/Amon/%s/gn/v%s/%s_Amon_PCMDI-test-1-0_piControl-withism_r3i1p1f1_gn_197901-199605.nc" % (
            var, today, var)
        f = cdms2.open(fn)
        s = f(var)
        if not numpy.allclose(s, data_ordered):
            raise "Error reordering: %s" % o
        else:
            print('order: %s, passed' % o)
        f.close()
print('Done')
# cmor.close()
print('Finito')



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import unittest
import signal

# ==============================
#  main thread
# ==============================
def run():
    unittest.main()


class TestCase(unittest.TestCase):

    def testTerminateSignal(self):
        self.assertEqual(cmor.get_terminate_signal(), -999)
        cmor.setup()
        self.assertEqual(cmor.get_terminate_signal(), signal.SIGTERM)
        cmor.set_terminate_signal(signal.SIGINT)
        self.assertEqual(cmor.get_terminate_signal(), signal.SIGINT)

if __name__ == '__main__':
    run()


import cmor
import unittest
from base_CMIP6_CV import BaseCVsTest


def run():
    unittest.main()


class TestTimeIntervalCheck(BaseCVsTest):

    def test_interval_too_big_in_axis(self):
        cmor.setup(inpath='Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.load_table("CMIP6_Amon.json")

        with self.assertRaises(cmor.CMORError):
            _ = cmor.axis(table_entry='time',
                          units='days since 2000-01-01 00:00:00',
                          coord_vals=[15, 45, 75],
                          cell_bounds=[0, 30, 60, 120])

        # Check for enhanced error message components
        self.assertCV("No frequency attribute provided in dataset configuration.")
        self.assertCV("30 days", "Expected interval between time axis values:")
        self.assertCV("45 days", "Actual interval between time axis values 1 and 2:")
        self.assertCV("Test/CMOR_input_example.json", "Input JSON:")
        self.assertCV("CMIP6_Amon.json", "Table JSON:")

    def test_interval_too_small_for_passed_time(self):
        cmor.setup(inpath='Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.load_table("CMIP6_Amon.json")

        itime = cmor.axis(table_entry='time',
                          units='days since 2000-01-01 00:00:00')
        ilat = cmor.axis(table_entry='latitude',
                         units='degrees_north',
                         coord_vals=[0],
                         cell_bounds=[-1, 1])
        ilon = cmor.axis(table_entry='longitude',
                         units='degrees_east',
                         coord_vals=[90],
                         cell_bounds=[89, 91])

        axis_ids = [itime, ilat, ilon]

        varid = cmor.variable('ts', 'K', axis_ids)

        _ = cmor.write(varid, [273, 273, 273], ntimes_passed=1,
                       time_vals=[15], time_bnds=[10, 20])

        with self.assertRaises(cmor.CMORError):
            _ = cmor.write(varid, [273, 273, 273], ntimes_passed=1,
                           time_vals=[35], time_bnds=[30, 40])

        # Check for enhanced error message components
        self.assertCV("Time interval mismatch detected for frequency: 'mon'")
        self.assertCV("30 days", "Expected interval between time axis values:")
        self.assertCV("20 days", "Actual interval between time axis values 0 and 1:")
        self.assertCV("Test/CMOR_input_example.json", "Input JSON:")
        self.assertCV("CMIP6_Amon.json", "Table JSON:")

    def test_interval_warning(self):
        cmor.setup(inpath='Tables',
                   netcdf_file_action=cmor.CMOR_REPLACE_4,
                   logfile=self.tmpfile)

        cmor.dataset_json("Test/CMOR_input_example.json")
        cmor.load_table("CMIP6_Amon.json")

        _ = cmor.axis(table_entry='time',
                      units='days since 2000-01-01 00:00:00',
                      coord_vals=[15, 45, 75],
                      cell_bounds=[0, 30, 60, 100])

        # Check for warning message
        self.assertCV("provided in dataset configuration.", "Warning: No frequency attribute")
        self.assertCV("30 days", "Expected interval between time axis values:")
        self.assertCV("35 days", "Actual interval between time axis values 1 and 2:")
        self.assertCV("Test/CMOR_input_example.json", "Input JSON:")
        self.assertCV("CMIP6_Amon.json", "Table JSON:")


if __name__ == '__main__':
    run()



#!/usr/bin/env python
import cmor
import numpy


def test_mode(mode, i, suffix=''):
    cmor.setup(inpath='Tables',
               netcdf_file_action=mode)
    cmor.dataset_json("Test/CMOR_input_example.json")

    table = 'CMIP6_Amon.json'
    cmor.load_table(table)
    levels = [100000.,
              92500.,
              85000.,
              70000.,
              60000.,
              50000.,
              40000.,
              30000.,
              25000.,
              20000.,
              15000.,
              10000.,
              7000.,
              5000.,
              3000.,
              2000.,
              1000.,
              999,
              998,
              997,
              996,
              995,
              994,
              500,
              100]

    axes = [{'table_entry': 'time',
             'units': 'months since 2000-01-01 00:00:00',
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            {'table_entry': 'plev19',
             'units': 'Pa',
             'coord_vals': levels},
            ]

    values = numpy.array(range(len(levels)), numpy.float32) + 195
    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    for var, units in (('ta', 'K'),):
        varid = cmor.variable(var,
                              units,
                              axis_ids,
                              history='variable history',
                              missing_value=-99
                              )
        print("Sending time bounds:", [[i, i + 1]])
        cmor.write(varid, values, time_vals=[i], time_bnds=[[i, i + 1]])

    fnm = cmor.close(varid, file_name=True)
    cmor.close()
    return fnm


fnm = ''
for i in range(5):
    print(i, fnm)
    if i == 0:
        mode = cmor.CMOR_REPLACE
    else:
        mode = cmor.CMOR_APPEND
    fnm = test_mode(cmor.CMOR_APPEND, i, fnm)
